module WeirdInteractor
  # Base interactor.
  class Base
    class UnregisteredEventCalled < StandardError; end

    attr_reader :params

    # @param params [ActionController::Parameters]
    def initialize(params)
      @params = params
    end

    # Registered callbacks for events.
    #
    # @return [Hash]
    def registered_events
      @registered_events ||= {}
    end

    # Register callback on event.
    #
    # @param event_names [Symbol, String, Array<Symbol>, Array<String>]
    # @param block       [Proc]
    def on(*event_names, &block)
      Array(event_names).each do |event_name|
        registered_events[event_name] = block
      end
    end

    # Execute callback for the given event.
    #
    # @param event_name [Symbol, String]
    # @param data       [Object]
    def publish(event_name, data)
      handler = registered_events[event_name]
      unless handler.nil?
        handler.call(data)
      else
        raise UnregisteredEventCalled, event_name
      end
    end
  end
end
